@@ -1,6 +1,6 @@
 /***************************************************************************
  *   KMidimon - ALSA sequencer based MIDI monitor                          *
- *   Copyright (C) 2005-2009 Pedro Lopez-Cabanillas                        *
+ *   Copyright (C) 2005-2010 Pedro Lopez-Cabanillas                        *
  *   plcl@users.sourceforge.net                                            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -19,36 +19,6 @@
  *   MA 02110-1301, USA                                                    *
  ***************************************************************************/
 
-#include <QMenu>
-#include <QEvent>
-#include <QContextMenuEvent>
-#include <QCursor>
-#include <QTreeView>
-#include <QTextStream>
-#include <QSignalMapper>
-#include <QVariant>
-#include <QToolTip>
-
-#include <klocale.h>
-#include <kaction.h>
-#include <kapplication.h>
-#include <kfiledialog.h>
-#include <kedittoolbar.h>
-#include <kglobal.h>
-#include <kglobalsettings.h>
-#include <ktoggleaction.h>
-#include <kstandardaction.h>
-#include <kactioncollection.h>
-#include <kxmlguifactory.h>
-#include <kurl.h>
-#include <ktabbar.h>
-#include <kicon.h>
-#include <kinputdialog.h>
-#include <kprogressdialog.h>
-#include <krecentfilesaction.h>
-#include <kmenubar.h>
-#include <kmessagebox.h>
-
 #include "kmidimon.h"
 #include "configdialog.h"
 #include "connectdlg.h"
@@ -58,8 +28,42 @@
 #include "sequenceradaptor.h"
 #include "slideraction.h"
 
+#include <KLocale>
+#include <KAction>
+#include <KApplication>
+#include <KFileDialog>
+#include <KEditToolBar>
+#include <KGlobal>
+#include <KGlobalSettings>
+#include <KToggleAction>
+#include <KStandardAction>
+#include <KActionCollection>
+#include <KXMLGUIFactory>
+#include <KUrl>
+#include <KTabBar>
+#include <KIcon>
+#include <KInputDialog>
+#include <KProgressDialog>
+#include <KRecentFilesAction>
+#include <KMenuBar>
+#include <KMessageBox>
+#include <KIO/NetAccess>
+
+#include <QMenu>
+#include <QEvent>
+#include <QContextMenuEvent>
+#include <QCursor>
+#include <QTreeView>
+#include <QTextStream>
+#include <QSignalMapper>
+#include <QVariant>
+#include <QToolTip>
+#include <QFileInfo>
+#include <QDropEvent>
+
 KMidimon::KMidimon() :
-    KXmlGuiWindow(0)
+    KXmlGuiWindow(0),
+    m_adaptor(0)
 {
     QWidget *vbox = new QWidget(this);
     QVBoxLayout *layout = new QVBoxLayout;
@@ -75,47 +79,60 @@ KMidimon::KMidimon() :
     m_view->setModel(m_proxy);
     m_view->setSortingEnabled(false);
     m_view->setSelectionMode(QAbstractItemView::SingleSelection);
+    setAcceptDrops(true);
     connect( m_view->selectionModel(),
              SIGNAL(currentChanged(const QModelIndex&, const QModelIndex&)),
              SLOT(slotCurrentChanged(const QModelIndex&, const QModelIndex&)) );
-    m_adaptor = new SequencerAdaptor(this);
-    m_adaptor->setModel(m_model);
-    m_adaptor->updateModelClients();
-    connect( m_model, SIGNAL(rowsInserted(QModelIndex,int,int)),
-                      SLOT(resizeColumns(QModelIndex,int,int)) );
-    connect( m_adaptor, SIGNAL(signalTicks(int)), SLOT(slotTicks(int)));
-    m_tabBar = new KTabBar(this);
-    m_tabBar->setWhatsThis(i18n("Track view selectors"));
-    m_tabBar->setShape(QTabBar::RoundedNorth);
+    try {
+        m_adaptor = new SequencerAdaptor(this);
+        m_adaptor->setModel(m_model);
+        m_adaptor->updateModelClients();
+        connect( m_model, SIGNAL(rowsInserted(QModelIndex,int,int)),
+                          SLOT(resizeColumns(QModelIndex,int,int)) );
+        connect( m_adaptor, SIGNAL(signalTicks(int)), SLOT(slotTicks(int)));
+        connect( m_adaptor, SIGNAL(finished()), SLOT(songFinished()));
+        m_tabBar = new KTabBar(this);
+        m_tabBar->setWhatsThis(i18n("Track view selectors"));
+        m_tabBar->setShape(QTabBar::RoundedNorth);
 #if QT_VERSION < 0x040500
-    m_tabBar->setTabReorderingEnabled(true);
-    m_tabBar->setCloseButtonEnabled(true);
-    connect( m_tabBar, SIGNAL(moveTab(int,int)),
-                       SLOT(reorderTabs(int,int)) );
-    connect( m_tabBar, SIGNAL(closeRequest(int)),
-                       SLOT(deleteTrack(int)) );
+        m_tabBar->setTabReorderingEnabled(true);
+        m_tabBar->setCloseButtonEnabled(true);
+        connect( m_tabBar, SIGNAL(moveTab(int,int)),
+                           SLOT(reorderTabs(int,int)) );
+        connect( m_tabBar, SIGNAL(closeRequest(int)),
+                           SLOT(deleteTrack(int)) );
 #else
-    m_tabBar->setExpanding(false);
-    m_tabBar->setMovable(true);
-    m_tabBar->setTabsClosable(true);
-    connect( m_tabBar, SIGNAL(tabCloseRequested(int)),
-                       SLOT(deleteTrack(int)) );
+        m_tabBar->setExpanding(false);
+        m_tabBar->setMovable(true);
+        m_tabBar->setTabsClosable(true);
+        connect( m_tabBar, SIGNAL(tabCloseRequested(int)),
+                           SLOT(deleteTrack(int)) );
 #endif
-    connect( m_tabBar, SIGNAL(newTabRequest()),
-                       SLOT(addTrack()) );
-    connect( m_tabBar, SIGNAL(tabDoubleClicked(int)),
-                       SLOT(changeTrack(int)) );
-    connect( m_tabBar, SIGNAL(currentChanged(int)),
-                       SLOT(tabIndexChanged(int)) );
-    layout->addWidget(m_tabBar);
-    layout->addWidget(m_view);
-    vbox->setLayout(layout);
-    setCentralWidget(vbox);
-    setupActions();
-    setAutoSaveSettings();
-    readConfiguration();
-    fileNew();
-    record();
+        connect( m_tabBar, SIGNAL(newTabRequest()),
+                           SLOT(addTrack()) );
+        connect( m_tabBar, SIGNAL(tabDoubleClicked(int)),
+                           SLOT(changeTrack(int)) );
+        connect( m_tabBar, SIGNAL(currentChanged(int)),
+                           SLOT(tabIndexChanged(int)) );
+        layout->addWidget(m_tabBar);
+        layout->addWidget(m_view);
+        vbox->setLayout(layout);
+        setCentralWidget(vbox);
+        setupActions();
+        setAutoSaveSettings();
+        readConfiguration();
+        fileNew();
+        record();
+    } catch (SequencerError& ex) {
+        QString errorstr = i18n("Fatal error from the ALSA sequencer. "
+            "This usually happens when the kernel doesn't have ALSA support, "
+            "or the device node (/dev/snd/seq) doesn't exists, "
+            "or the kernel module (snd_seq) is not loaded. "
+            "Please check your ALSA/MIDI configuration. Returned error was: %1",
+            ex.qstrError());
+        KMessageBox::error(0, errorstr, i18n("Error"));
+        close();
+    }
 }
 
 void KMidimon::setupActions()
@@ -123,11 +140,12 @@ void KMidimon::setupActions()
     const QString columnName[COLUMN_COUNT] = {
             i18n("Ticks"),
             i18n("Time"),
-            i18n("Source"),
-            i18n("Event Kind"),
+            i18nc("event origin", "Source"),
+            i18nc("type of event", "Event Kind"),
             i18n("Channel"),
             i18n("Data 1"),
-            i18n("Data 2")
+            i18n("Data 2"),
+            i18n("Data 3")
     };
     const QString actionName[COLUMN_COUNT] = {
             "show_ticks",
@@ -136,7 +154,8 @@ void KMidimon::setupActions()
             "show_kind",
             "show_channel",
             "show_data1",
-            "show_data2"
+            "show_data2",
+            "show_data3"
     };
     KAction* a;
 
@@ -144,7 +163,7 @@ void KMidimon::setupActions()
     a->setWhatsThis(i18n("Exit the application"));
     a = KStandardAction::open(this, SLOT(fileOpen()), actionCollection());
     a->setWhatsThis(i18n("Open a disk file"));
-    m_recentFiles = KStandardAction::openRecent(this, SLOT(slotURLSelected(const KUrl&)), actionCollection());
+    m_recentFiles = KStandardAction::openRecent(this, SLOT(openURL(const KUrl&)), actionCollection());
     a = KStandardAction::openNew(this, SLOT(fileNew()), actionCollection());
     a->setWhatsThis(i18n("Clear the current data and start a new empty session"));
     m_save = KStandardAction::saveAs(this, SLOT(fileSave()), actionCollection());
@@ -184,14 +203,14 @@ void KMidimon::setupActions()
     actionCollection()->addAction("pause", m_pause);
 
     m_forward = new KAction(this);
-    m_forward->setText(i18n("Forward"));
+    m_forward->setText(i18nc("player skip forward", "Forward"));
     m_forward->setIcon(KIcon("media-skip-forward"));
     m_forward->setWhatsThis(i18n("Move the playback position to the last event"));
     connect(m_forward, SIGNAL(triggered()), SLOT(forward()));
     actionCollection()->addAction("forward", m_forward);
 
     m_rewind = new KAction(this);
-    m_rewind->setText(i18n("Backward"));
+    m_rewind->setText(i18nc("player skip backward", "Backward"));
     m_rewind->setIcon(KIcon("media-skip-backward"));
     m_rewind->setWhatsThis(i18n("Move the playback position to the first event"));
     connect(m_rewind, SIGNAL(triggered()), SLOT(rewind()));
@@ -284,6 +303,18 @@ void KMidimon::setupActions()
     connect(m_tempo100, SIGNAL(triggered()), this, SLOT(tempoReset()));
     actionCollection()->addAction("tempo100", m_tempo100);
 
+    m_loop = new KToggleAction(this);
+    m_loop->setText(i18n("Player Loop"));
+    m_loop->setWhatsThis(i18n("Start playing again at song ending"));
+    connect(m_loop, SIGNAL(triggered()), SLOT(slotLoop()));
+    actionCollection()->addAction("loop", m_loop);
+
+    m_muteTrack = new KToggleAction(this);
+    m_muteTrack->setText(i18n("Mute Track"));
+    m_muteTrack->setWhatsThis(i18n("Mute (silence) the track"));
+    connect(m_muteTrack, SIGNAL(triggered()), SLOT(muteCurrentTrack()));
+    actionCollection()->addAction("mute_track", m_muteTrack );
+
     setStandardToolBarMenuEnabled(true);
     setupGUI();
 
@@ -317,55 +348,58 @@ void KMidimon::fileNew()
     m_adaptor->rewind();
     tempoReset();
     updateView();
+    updateCaption();
 }
 
-void KMidimon::slotURLSelected(const KUrl& url)
+void KMidimon::openURL(const KUrl& url)
 {
-    QString path = url.toLocalFile();
-    if (!path.isNull()) {
-        QFileInfo finfo(path);
-        if (finfo.exists()) {
-            try {
-                m_file = path;
-                m_view->blockSignals(true);
-                stop();
-                m_model->clear();
-                for (int i = m_tabBar->count() - 1; i >= 0; i--) {
-                    m_tabBar->removeTab(i);
-                }
-                m_pd = new KProgressDialog(this, i18n("Load MIDI file"),
-                                i18n("Loading..."));
-                m_pd->setAllowCancel(false);
-                m_pd->setMinimumDuration(500);
-                m_pd->progressBar()->setRange(0, finfo.size());
-                m_pd->progressBar()->setValue(0);
-                connect( m_model, SIGNAL(loadProgress(int)),
-                         m_pd->progressBar(), SLOT(setValue(int)) );
-                m_model->loadFromFile(path);
-                m_pd->progressBar()->setValue(finfo.size());
-                m_adaptor->setResolution(m_model->getSMFDivision());
-                if (m_model->getInitialTempo() > 0) {
-                    m_adaptor->setTempo(m_model->getInitialTempo());
-                    m_adaptor->queue_set_tempo();
-                }
-                m_adaptor->rewind();
-                int ntrks = m_model->getSMFTracks();
-                if (ntrks < 1) ntrks = 1;
-                for (int i = 0; i < ntrks; i++)
-                    addNewTab(i+1);
-                m_tabBar->setCurrentIndex(0);
-                m_proxy->setFilterTrack(0);
-                m_model->setCurrentTrack(0);
-                for (int i = 0; i < COLUMN_COUNT; ++i)
-                    m_view->resizeColumnToContents(i);
-                tempoReset();
-            } catch (...) {
-                m_model->clear();
+    QString tmpFile;
+    if(KIO::NetAccess::download(url, tmpFile, this)) {
+        try {
+            QFileInfo finfo(tmpFile);
+            m_file = url.toLocalFile();
+            if (m_file.isEmpty())
+                m_file = url.prettyUrl();
+            m_view->blockSignals(true);
+            stop();
+            m_model->clear();
+            for (int i = m_tabBar->count() - 1; i >= 0; i--) {
+                m_tabBar->removeTab(i);
             }
-            m_view->blockSignals(false);
-            m_recentFiles->addUrl(url);
-            delete m_pd;
+            m_pd = new KProgressDialog(this, i18n("Load file"),
+                            i18n("Loading..."));
+            m_pd->setAllowCancel(false);
+            m_pd->setMinimumDuration(500);
+            m_pd->progressBar()->setRange(0, finfo.size());
+            m_pd->progressBar()->setValue(0);
+            connect( m_model, SIGNAL(loadProgress(int)),
+                     m_pd->progressBar(), SLOT(setValue(int)) );
+            m_model->loadFromFile(tmpFile);
+            m_pd->progressBar()->setValue(finfo.size());
+            m_adaptor->setResolution(m_model->getSMFDivision());
+            if (m_model->getInitialTempo() > 0) {
+                m_adaptor->setTempo(m_model->getInitialTempo());
+                m_adaptor->queue_set_tempo();
+            }
+            m_adaptor->rewind();
+            int ntrks = m_model->getSMFTracks();
+            if (ntrks < 1) ntrks = 1;
+            for (int i = 0; i < ntrks; i++)
+                addNewTab(m_model->getTrackForIndex(i));
+            m_tabBar->setCurrentIndex(0);
+            m_proxy->setFilterTrack(0);
+            m_model->setCurrentTrack(0);
+            for (int i = 0; i < COLUMN_COUNT; ++i)
+                m_view->resizeColumnToContents(i);
+            tempoReset();
+        } catch (...) {
+            m_model->clear();
         }
+        m_view->blockSignals(false);
+        m_recentFiles->addUrl(url);
+        KIO::NetAccess::removeTempFile(tmpFile);
+        updateCaption();
+        delete m_pd;
     }
 }
 
@@ -373,10 +407,9 @@ void KMidimon::fileOpen()
 {
     KUrl u = KFileDialog::getOpenUrl(
             KUrl("kfiledialog:///MIDIMONITOR"),
-            i18n("*.mid *.midi *.kar|MIDI files (*.mid)"),
-            this,
+            "audio/midi audio/cakewalk", this,
             i18n("Open MIDI file"));
-    if (!u.isEmpty()) slotURLSelected(u);
+    if (!u.isEmpty()) openURL(u);
 }
 
 void KMidimon::fileSave()
@@ -406,6 +439,7 @@ void KMidimon::saveConfiguration()
 {
     int i;
     KConfigGroup config = KGlobal::config()->group("Settings");
+    if (m_adaptor == NULL) return;
     config.writeEntry("resolution", m_defaultResolution);
     config.writeEntry("tempo", m_defaultTempo);
     config.writeEntry("alsa", m_proxy->showAlsaMsg());
@@ -419,6 +453,7 @@ void KMidimon::saveConfiguration()
     config.writeEntry("translate_notes", m_model->translateNotes());
     config.writeEntry("translate_ctrls", m_model->translateCtrls());
     config.writeEntry("instrument", m_model->getInstrumentName());
+    config.writeEntry("encoding", m_model->getEncoding());
     config.writeEntry("fixed_font", getFixedFont());
     for (i = 0; i < COLUMN_COUNT; ++i) {
         config.writeEntry(QString("show_column_%1").arg(i),
@@ -450,6 +485,7 @@ void KMidimon::readConfiguration()
     m_model->setTranslateNotes(config.readEntry("translate_notes", false));
     m_model->setTranslateCtrls(config.readEntry("translate_ctrls", false));
     m_model->setInstrumentName(config.readEntry("instrument", QString()));
+    m_model->setEncoding(config.readEntry("encoding", QString()));
     m_defaultResolution = config.readEntry("resolution", RESOLUTION);
     m_defaultTempo = config.readEntry("tempo", TEMPO_BPM);
     m_adaptor->setResolution(m_defaultResolution);
@@ -473,60 +509,67 @@ void KMidimon::preferences()
 {
     int i;
     bool was_running;
-    ConfigDialog dlg;
-
-    dlg.setTempo(m_defaultTempo);
-    dlg.setResolution(m_defaultResolution);
-    dlg.setRegAlsaMsg(m_proxy->showAlsaMsg());
-    dlg.setRegChannelMsg(m_proxy->showChannelMsg());
-    dlg.setRegCommonMsg(m_proxy->showCommonMsg());
-    dlg.setRegRealTimeMsg(m_proxy->showRealTimeMsg());
-    dlg.setRegSysexMsg(m_proxy->showSysexMsg());
-    dlg.setRegSmfMsg(m_proxy->showSmfMsg());
-    dlg.setShowClientNames(m_model->showClientNames());
-    dlg.setTranslateSysex(m_model->translateSysex());
-    dlg.setTranslateNotes(m_model->translateNotes());
-    dlg.setTranslateCtrls(m_model->translateCtrls());
-    dlg.setInstruments(m_model->getInstruments());
-    dlg.setInstrumentName(m_model->getInstrumentName());
-    dlg.setUseFixedFont(getFixedFont());
+    QPointer<ConfigDialog> dlg = new ConfigDialog(this);
+    dlg->setTempo(m_defaultTempo);
+    dlg->setResolution(m_defaultResolution);
+    dlg->setRegAlsaMsg(m_proxy->showAlsaMsg());
+    dlg->setRegChannelMsg(m_proxy->showChannelMsg());
+    dlg->setRegCommonMsg(m_proxy->showCommonMsg());
+    dlg->setRegRealTimeMsg(m_proxy->showRealTimeMsg());
+    dlg->setRegSysexMsg(m_proxy->showSysexMsg());
+    dlg->setRegSmfMsg(m_proxy->showSmfMsg());
+    dlg->setShowClientNames(m_model->showClientNames());
+    dlg->setTranslateSysex(m_model->translateSysex());
+    dlg->setTranslateNotes(m_model->translateNotes());
+    dlg->setTranslateCtrls(m_model->translateCtrls());
+    dlg->setInstruments(m_model->getInstruments());
+    dlg->setInstrumentName(m_model->getInstrumentName());
+    dlg->setEncoding(m_model->getEncoding());
+    dlg->setUseFixedFont(getFixedFont());
     for (i = 0; i < COLUMN_COUNT; ++i) {
-        dlg.setShowColumn(i, m_popupAction[i]->isChecked());
+        dlg->setShowColumn(i, m_popupAction[i]->isChecked());
     }
-    if (dlg.exec()) {
-        was_running = m_adaptor->isRecording();
-        if (was_running) stop();
-        m_proxy->setFilterAlsaMsg(dlg.isRegAlsaMsg());
-        m_proxy->setFilterChannelMsg(dlg.isRegChannelMsg());
-        m_proxy->setFilterCommonMsg(dlg.isRegCommonMsg());
-        m_proxy->setFilterRealTimeMsg(dlg.isRegRealTimeMsg());
-        m_proxy->setFilterSysexMsg(dlg.isRegSysexMsg());
-        m_proxy->setFilterSmfMsg(dlg.isRegSmfMsg());
-        m_model->setShowClientNames(dlg.showClientNames());
-        m_model->setTranslateSysex(dlg.translateSysex());
-        m_model->setTranslateNotes(dlg.translateNotes());
-        m_model->setTranslateCtrls(dlg.translateCtrls());
-        m_model->setInstrumentName(dlg.getInstrumentName());
-        m_defaultTempo = dlg.getTempo();
-        m_defaultResolution = dlg.getResolution();
-        setFixedFont(dlg.useFixedFont());
-        for (i = 0; i < COLUMN_COUNT; ++i) {
-            setColumnStatus(i, dlg.showColumn(i));
+    if (dlg->exec() == QDialog::Accepted) {
+        if (dlg != NULL) {
+            was_running = m_adaptor->isRecording();
+            if (was_running) stop();
+            m_proxy->setFilterAlsaMsg(dlg->isRegAlsaMsg());
+            m_proxy->setFilterChannelMsg(dlg->isRegChannelMsg());
+            m_proxy->setFilterCommonMsg(dlg->isRegCommonMsg());
+            m_proxy->setFilterRealTimeMsg(dlg->isRegRealTimeMsg());
+            m_proxy->setFilterSysexMsg(dlg->isRegSysexMsg());
+            m_proxy->setFilterSmfMsg(dlg->isRegSmfMsg());
+            m_model->setShowClientNames(dlg->showClientNames());
+            m_model->setTranslateSysex(dlg->translateSysex());
+            m_model->setTranslateNotes(dlg->translateNotes());
+            m_model->setTranslateCtrls(dlg->translateCtrls());
+            m_model->setInstrumentName(dlg->getInstrumentName());
+            m_model->setEncoding(dlg->getEncoding());
+            m_defaultTempo = dlg->getTempo();
+            m_defaultResolution = dlg->getResolution();
+            setFixedFont(dlg->useFixedFont());
+            for (i = 0; i < COLUMN_COUNT; ++i) {
+                setColumnStatus(i, dlg->showColumn(i));
+            }
+            if (was_running) record();
         }
-        if (was_running) record();
     }
+    delete dlg;
 }
 
 void KMidimon::record()
 {
     m_adaptor->forward();
     m_adaptor->record();
-    updateState("recording_state", i18n("recording"));
+    if (m_adaptor->isRecording())
+        updateState("recording_state", i18n("recording"));
 }
 
 void KMidimon::stop()
 {
-    if (m_adaptor->isRecording() | m_adaptor->isPlaying()) {
+    if ( m_adaptor->isRecording() ||
+         m_adaptor->isPlaying() ||
+         m_adaptor->isPaused()) {
         m_adaptor->stop();
         songFinished();
     }
@@ -534,19 +577,22 @@ void KMidimon::stop()
 
 void KMidimon::songFinished()
 {
-    updateState("stopped_state", i18n("stopped"));
+    updateState("stopped_state", i18nc("player stopped","stopped"));
     updateView();
 }
 
 void KMidimon::play()
 {
     m_adaptor->play();
-    updateState("playing_state", i18n("playing"));
+    if (m_adaptor->isPlaying())
+        updateState("playing_state", i18nc("player playing","playing"));
 }
 
 void KMidimon::pause()
 {
     m_adaptor->pause(m_pause->isChecked());
+    //if (m_adaptor->isPaused())
+    //  updateState("paused_state", i18nc("player paused","paused"));
 }
 
 void KMidimon::rewind()
@@ -561,18 +607,30 @@ void KMidimon::forward()
     updateView();
 }
 
+void KMidimon::updateCaption()
+{
+    QFileInfo finfo(m_file);
+    QString name = finfo.fileName();
+    if (name.isEmpty())
+        name = i18n("(empty)");
+    setCaption(i18n("%1 [%2]", name, m_currentState));
+}
+
 void KMidimon::updateState(const QString newState, const QString stateName)
 {
-    setCaption(i18n("ALSA MIDI Monitor [%1]", stateName));
+    m_currentState = stateName;
+    updateCaption();
     slotStateChanged(newState);
+    m_pause->setChecked(m_adaptor->isPaused());
 }
 
 void KMidimon::editToolbars()
 {
-    KEditToolBar dlg(actionCollection());
-    if (dlg.exec()) {
-        setupGUI();
+    QPointer<KEditToolBar> dlg = new KEditToolBar(actionCollection());
+    if (dlg->exec() == QDialog::Accepted) {
+        if (dlg != NULL) setupGUI();
     }
+    delete dlg;
 }
 
 void KMidimon::connectAll()
@@ -591,27 +649,30 @@ void KMidimon::configConnections()
     QStringList subs = m_adaptor->list_subscribers();
     QStringList outputs = m_adaptor->outputConnections();
     m_outputConn = m_adaptor->output_subscriber();
-    ConnectDlg dlg(this, inputs, subs, outputs, m_outputConn);
-    if (dlg.exec()) {
-        QStringList desired = dlg.getSelectedInputs();
-        subs = m_adaptor->list_subscribers();
-        QStringList::ConstIterator i;
-        for (i = subs.constBegin(); i != subs.constEnd(); ++i) {
-            if (desired.contains(*i) == 0) {
-                m_adaptor->disconnect_input(*i);
+    QPointer<ConnectDlg> dlg = new ConnectDlg(this, inputs, subs, outputs, m_outputConn);
+    if (dlg->exec() == QDialog::Accepted) {
+        if (dlg != NULL) {
+            QStringList desired = dlg->getSelectedInputs();
+            subs = m_adaptor->list_subscribers();
+            QStringList::ConstIterator i;
+            for (i = subs.constBegin(); i != subs.constEnd(); ++i) {
+                if (desired.contains(*i) == 0) {
+                    m_adaptor->disconnect_input(*i);
+                }
             }
-        }
-        for (i = desired.constBegin(); i != desired.constEnd(); ++i) {
-            if (subs.contains(*i) == 0) {
-                m_adaptor->connect_input(*i);
+            for (i = desired.constBegin(); i != desired.constEnd(); ++i) {
+                if (subs.contains(*i) == 0) {
+                    m_adaptor->connect_input(*i);
+                }
+            }
+            QString newOut = dlg->getSelectedOutput();
+            if (newOut != m_outputConn) {
+                m_adaptor->disconnect_output(m_outputConn);
+                m_adaptor->connect_output(m_outputConn = newOut);
             }
-        }
-        QString newOut = dlg.getSelectedOutput();
-        if (newOut != m_outputConn) {
-            m_adaptor->disconnect_output(m_outputConn);
-            m_adaptor->connect_output(m_outputConn = newOut);
         }
     }
+    delete dlg;
 }
 
 void KMidimon::setColumnStatus(int colNum, bool status)
@@ -631,6 +692,9 @@ void KMidimon::toggleColumn(int colNum)
 void KMidimon::contextMenuEvent(QContextMenuEvent*)
 {
     Q_CHECK_PTR( m_popup );
+    QVariant data = m_tabBar->tabData(m_tabBar->currentIndex());
+    int track = data.toInt() - 1;
+    m_muteTrack->setChecked(m_model->getSong()->mutedState(track));
     m_popup->exec(QCursor::pos());
 }
 
@@ -657,17 +721,15 @@ void KMidimon::resizeAllColumns()
 
 void KMidimon::addNewTab(int data)
 {
-    QString tabName = i18n("Track %1", data);
+    QString tabName = i18nc("song track", "Track %1", data);
     int i = m_tabBar->addTab(tabName);
     m_tabBar->setTabData(i, QVariant(data));
     m_tabBar->setTabWhatsThis(i, i18n("Track %1 View Selector", data));
-    //qDebug() << "new tab data: " << data;
 }
 
 void KMidimon::tabIndexChanged(int index)
 {
     QVariant data = m_tabBar->tabData(index);
-    //qDebug() << "current tab data: " << data.toInt();
     m_proxy->setFilterTrack(data.toInt()-1);
     m_model->setCurrentTrack(data.toInt()-1);
 }
@@ -700,8 +762,7 @@ void KMidimon::changeTrack(int tabIndex)
 {
     int track = m_tabBar->tabData(tabIndex).toInt();
     if (askTrackFilter(track)) {
-        //qDebug() << "changeTrack: " << tabIndex;
-        QString tabName = i18n("Track %1", track);
+        QString tabName = i18nc("song track", "Track %1", track);
         m_tabBar->setTabData(tabIndex, track);
         m_tabBar->setTabText(tabIndex, tabName);
         m_tabBar->setTabWhatsThis(tabIndex, i18n("Track %1 View Selector", track));
@@ -721,7 +782,6 @@ void KMidimon::changeCurrentTrack()
 
 void KMidimon::reorderTabs(int fromIndex, int toIndex)
 {
-    //qDebug() << "reorderTabs(" << fromIndex << "," << toIndex << ")";
     QIcon icon = m_tabBar->tabIcon(fromIndex);
     QString text = m_tabBar->tabText(fromIndex);
     QVariant data = m_tabBar->tabData(fromIndex);
@@ -765,22 +825,27 @@ KMidimon::songFileInfo()
     QString infostr;
     if (m_file.isEmpty())
         infostr = i18n("No file loaded");
-    else
-        infostr = i18n("File: <b>%1</b><br>"
-                       "SMF Format: <b>%2</b><br>"
-                       "Number of tracks: <b>%3</b><br>"
-                       "Number of events: <b>%4</b><br>"
-                       "Division: <b>%5 ppq</b><br>"
-                       "Initial tempo: <b>%6 bpm</b><br>"
-                       "Duration: <b>%7</b>",
-                       m_file,
-                       m_model->getSMFFormat(),
+    else {
+        QFileInfo finfo(m_file);
+        infostr = i18n("File: <b>%1</b><br/>"
+                       "Created: <b>%2</b><br/>"
+                       "Modified: <b>%3</b><br/>"
+                       "Format: <b>%4</b><br/>"
+                       "Number of tracks: <b>%5</b><br/>"
+                       "Number of events: <b>%6</b><br/>"
+                       "Division: <b>%7 ppq</b><br/>"
+                       "Initial tempo: <b>%8 bpm</b><br/>"
+                       "Duration: <b>%9</b>",
+                       finfo.fileName(),
+                       finfo.created().toString(Qt::DefaultLocaleLongDate),
+                       finfo.lastModified().toString(Qt::DefaultLocaleLongDate),
+                       m_model->getFileFormat(),
                        m_model->getSMFTracks(),
                        m_model->getSong()->size(),
                        m_model->getSMFDivision(),
                        m_model->getInitialTempo(),
-                       m_model->getDuration()
-                       );
+                       m_model->getDuration() );
+    }
     KMessageBox::information(this, infostr, i18n("Sequence Information"));
 }
 
@@ -800,3 +865,42 @@ void KMidimon::tempoSlider(int value)
     m_tempoSlider->slider()->setToolTip(tip);
     QToolTip::showText(QCursor::pos(), tip, this);
 }
+
+void KMidimon::muteTrack(int tabIndex)
+{
+    QVariant data = m_tabBar->tabData(tabIndex);
+    int track = data.toInt() - 1;
+    Song* song = m_model->getSong();
+    if (song != NULL) {
+        bool newState = !song->mutedState(track);
+        song->setMutedState(track, newState);
+        if (newState) m_adaptor->removeTrackEvents(track);
+    }
+}
+
+void KMidimon::muteCurrentTrack()
+{
+    muteTrack(m_tabBar->currentIndex());
+}
+
+void KMidimon::slotLoop()
+{
+    m_adaptor->setLoop(m_loop->isChecked());
+}
+
+void KMidimon::dragEnterEvent( QDragEnterEvent * event )
+{
+    if (event->mimeData()->hasUrls()) {
+        event->acceptProposedAction();
+    }
+}
+
+void KMidimon::dropEvent( QDropEvent * event )
+{
+    if ( event->mimeData()->hasUrls() ) {
+        QList<QUrl> urls = event->mimeData()->urls();
+        if (!urls.empty())
+            openURL(urls.first());
+        event->accept();
+    }
+}
