@@ -1,6 +1,6 @@
 /***************************************************************************
  *   KMidimon - ALSA sequencer based MIDI monitor                          *
- *   Copyright (C) 2005-2009 Pedro Lopez-Cabanillas                        *
+ *   Copyright (C) 2005-2010 Pedro Lopez-Cabanillas                        *
  *   plcl@users.sourceforge.net                                            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -19,8 +19,8 @@
  *   MA 02110-1301, USA                                                    *
  ***************************************************************************/
 
-#ifndef _KMIDIMON_H_
-#define _KMIDIMON_H_
+#ifndef KMIDIMON_H
+#define KMIDIMON_H
 
 #include <QPointer>
 #include <kxmlguiwindow.h>
@@ -45,7 +45,7 @@ class SequenceModel;
 class ProxyModel;
 class KPlayerPopupSliderAction;
 
-const int COLUMN_COUNT = 7;
+const int COLUMN_COUNT = 8;
 
 class KMidimon : public KXmlGuiWindow
 {
@@ -70,12 +70,15 @@ public slots:
     void addTrack();
     void deleteCurrentTrack();
     void changeCurrentTrack();
+    void muteCurrentTrack();
     void deleteTrack(int tabIndex);
     void changeTrack(int tabIndex);
+    void muteTrack(int tabIndex);
     void songFinished();
-    void slotURLSelected(const KUrl& url);
+    void openURL(const KUrl& url);
     void tempoReset();
     void tempoSlider(int value);
+    void slotLoop();
 
     void disconnectAll();
     void configConnections();
@@ -92,6 +95,8 @@ public slots:
     void slotCurrentChanged(const QModelIndex& curr, const QModelIndex& prev);
     void updateView();
     void songFileInfo();
+    void dropEvent( QDropEvent * event );
+    void dragEnterEvent( QDragEnterEvent * event );
 
 protected:
     void saveConfiguration();
@@ -101,6 +106,7 @@ protected:
     bool getFixedFont() const { return m_useFixedFont; }
     void addNewTab(int data);
     bool askTrackFilter(int& track);
+    void updateCaption();
 
 private:
     SequencerAdaptor *m_adaptor;
@@ -133,10 +139,13 @@ private:
     QPointer<KProgressDialog> m_pd;
     QString m_outputConn;
     EventFilter* m_filter;
+    KToggleAction *m_loop;
+    KToggleAction *m_muteTrack;
 	bool m_useFixedFont;
     int m_defaultTempo;
     int m_defaultResolution;
     QString m_file;
+    QString m_currentState;
 };
 
-#endif // _KMIDIMON_H_
+#endif // KMIDIMON_H
